@@ -1,7 +1,7 @@
 /*
  * tclUnixFCmd.c
  *
- *	This file implements the unix specific portion of file manipulation
+ *	This file implements the Unix specific portion of file manipulation
  *	subcommands of the "file" command. All filename arguments should
  *	already be translated to native format.
  *
@@ -350,7 +350,7 @@ DoRenameFile(
     }
 
     /*
-     * IRIX returns EIO when you attept to move a directory into itself. We
+     * IRIX returns EIO when you attempt to move a directory into itself. We
      * just map EIO to EINVAL get the right message on SGI. Most platforms
      * don't return EIO except in really strange cases.
      */
@@ -740,7 +740,7 @@ DoCreateDirectory(
  *
  *	Recursively copies a directory. The target directory dst must not
  *	already exist. Note that this function does not merge two directory
- *	hierarchies, even if the target directory is an an empty directory.
+ *	hierarchies, even if the target directory is an empty directory.
  *
  * Results:
  *	If the directory was successfully copied, returns TCL_OK. Otherwise
@@ -789,8 +789,7 @@ TclpObjCopyDirectory(
     Tcl_DStringFree(&dstString);
 
     if (ret != TCL_OK) {
-	*errorPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
-	Tcl_DStringFree(&ds);
+	*errorPtr = TclDStringToObj(&ds);
 	Tcl_IncrRefCount(*errorPtr);
     }
     return ret;
@@ -843,8 +842,7 @@ TclpObjRemoveDirectory(
     Tcl_DStringFree(&pathString);
 
     if (ret != TCL_OK) {
-	*errorPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
-	Tcl_DStringFree(&ds);
+	*errorPtr = TclDStringToObj(&ds);
 	Tcl_IncrRefCount(*errorPtr);
     }
     return ret;
